/// <reference types="node" />
import type { SecretsManager, Event, SerializableMap, Serializable } from '@resolve-js/core';
import stream from 'stream';
import { MAINTENANCE_MODE_AUTO, MAINTENANCE_MODE_MANUAL } from './constants';
import * as t from 'io-ts';
import * as iotsTypes from 'io-ts-types';
export declare function validate<T extends t.Type<any>>(schema: T, params: any, errorName?: string): t.TypeOf<T>;
declare type DefinedType<T extends any> = undefined extends Extract<T, undefined> ? Exclude<T, undefined> : T;
export declare type UnbrandProps<T extends any> = {
    [Property in keyof T]: DefinedType<T[Property]> extends string & t.Brand<infer B> ? string | Extract<T[Property], undefined> : DefinedType<T[Property]> extends t.Branded<infer S, infer B> ? S | Extract<T[Property], undefined> : T[Property];
};
export declare type InputEvent = Omit<Event, 'payload'> & {
    payload?: Serializable;
};
export declare type VersionlessEvent = Omit<InputEvent, 'aggregateVersion'>;
export declare type EventThreadData = {
    threadCounter: number;
    threadId: number;
};
export declare type SavedEvent = Event & EventThreadData & SerializableMap;
export declare type OldEvent = Event;
export declare type ReplicationStatus = 'batchInProgress' | 'batchDone' | 'error' | 'notStarted' | 'serviceError';
export declare type ReplicationState = {
    status: ReplicationStatus;
    statusData: SerializableMap | null;
    paused: boolean;
    iterator: SerializableMap | null;
    successEvent: OldEvent | null;
};
export declare function getInitialReplicationState(): ReplicationState;
export declare type EventStoreDescription = {
    eventCount: number;
    secretCount: number;
    setSecretCount: number;
    deletedSecretCount: number;
    isFrozen: boolean;
    lastEventTimestamp: number;
    cursor?: string;
    resourceNames?: {
        [key: string]: string;
    };
};
export declare type CheckForResourceError = (errors: Error[]) => void;
declare type DeleteSecret = SecretsManager['deleteSecret'];
declare type GetSecret = SecretsManager['getSecret'];
declare type SetSecret = SecretsManager['setSecret'];
declare type ShapeEvent = (event: any, additionalFields?: any) => SavedEvent;
export declare type ValidateEventFilter = (filter: EventFilter) => void;
export declare type Cursor = string | null;
export declare type GetNextCursor = (prevCursor: Cursor, events: EventThreadData[]) => string;
export declare type EventsWithCursor = {
    cursor: Cursor;
    events: SavedEvent[];
};
export declare type EventWithCursor = {
    cursor: Cursor;
    event: SavedEvent;
};
declare const TimestampFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>]>;
declare type TimestampFilterChecked = t.TypeOf<typeof TimestampFilterSchema>;
export declare type TimestampFilter = UnbrandProps<TimestampFilterChecked>;
declare const CursorFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.TypeC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
declare type CursorFilterChecked = t.TypeOf<typeof CursorFilterSchema>;
export declare type CursorFilter = UnbrandProps<CursorFilterChecked>;
declare const EventFilterSchemaSimple: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
export declare const EventFilterSchema: t.Type<UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventsSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventsSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, unknown>;
declare type EventFilterChecked = t.TypeOf<typeof EventFilterSchemaSimple>;
export declare type EventFilter = UnbrandProps<EventFilterChecked>;
export declare type LatestEventFilter = Omit<EventFilter, 'limit' | 'eventsSizeLimit'>;
export declare type SecretFilter = {
    idx?: SecretRecord['idx'] | null;
    skip?: number;
    limit: number;
    ids?: Array<SecretRecord['id']> | null;
    includeDeleted?: boolean;
};
export declare type SecretsWithIdx = {
    idx: SecretRecord['idx'] | null;
    secrets: SecretRecord[];
};
export declare type SecretRecord = {
    idx: number;
    id: string;
    secret: string | null;
};
export declare type GatheredSecrets = {
    existingSecrets: SecretRecord[];
    deletedSecrets: Array<SecretRecord['id']>;
};
export declare type OldSecretRecord = SecretRecord;
export declare function isTimestampFilter(filter: EventFilter): filter is TimestampFilter;
export declare function isCursorFilter(filter: EventFilter): filter is CursorFilter;
export declare type RemoveFirstType<T extends any[]> = T extends [infer _, ...infer R] ? R : never;
export declare type PromiseResultType<T extends any> = T extends Promise<infer R> ? R : T;
export declare type PoolMethod<ConnectedProps extends AdapterPoolConnectedProps, M extends (...args: any) => any> = (pool: AdapterPoolConnected<ConnectedProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare type UnconnectedPoolMethod<ConnectedProps extends AdapterPoolConnectedProps, M extends (...args: any) => any> = (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare const AdapterConfigSchema: t.PartialC<{
    snapshotBucketSize: t.BrandC<t.NumberC, t.IntBrand>;
}>;
declare type AdapterConfigChecked = t.TypeOf<typeof AdapterConfigSchema>;
export declare type AdapterConfig = UnbrandProps<AdapterConfigChecked>;
export declare const AdapterTableNamesSchema: t.PartialC<{
    secretsTableName: iotsTypes.NonEmptyStringC;
    eventsTableName: iotsTypes.NonEmptyStringC;
    snapshotsTableName: iotsTypes.NonEmptyStringC;
    subscribersTableName: iotsTypes.NonEmptyStringC;
}>;
declare type AdapterTableNamesChecked = t.TypeOf<typeof AdapterTableNamesSchema>;
export declare type AdapterTableNames = UnbrandProps<AdapterTableNamesChecked>;
export declare type AdapterTableNamesProps = Required<AdapterTableNames>;
export declare type AdapterPoolPrimalProps = {
    disposed: boolean;
    validateEventFilter: ValidateEventFilter;
    isConnected: boolean;
    createGetConnectPromise: () => () => Promise<void>;
    getConnectPromise: () => Promise<void>;
    maybeThrowResourceError: CheckForResourceError;
    bucketSize: number;
    counters: Map<string, number>;
    getNextCursor: GetNextCursor;
    getVacantTimeInMillis?: () => number;
};
export declare type AdapterPoolPrivateConnectedProps = {
    injectEvent: (event: SavedEvent) => Promise<void>;
    injectEvents: (events: SavedEvent[]) => Promise<void>;
    injectSecret?: (secretRecord: SecretRecord) => Promise<void>;
    loadEventsByTimestamp: (filter: TimestampFilter) => Promise<EventsWithCursor>;
    loadEventsByCursor: (filter: CursorFilter) => Promise<EventsWithCursor>;
    deleteSecret: DeleteSecret;
    getSecret: GetSecret;
    setSecret: SetSecret;
    waitConnect: () => Promise<void>;
    shapeEvent: ShapeEvent;
    initEvents: () => Promise<any[]>;
    initSecrets: () => Promise<any[]>;
    initFinal: () => Promise<any[]>;
    dropEvents: () => Promise<any[]>;
    dropSecrets: () => Promise<any[]>;
    dropFinal: () => Promise<any[]>;
};
export declare type AdapterPoolConnectedProps = Adapter & AdapterPoolPrivateConnectedProps;
export declare type AdapterPoolPossiblyUnconnected<ConnectedProps extends AdapterPoolConnectedProps> = AdapterPoolPrimalProps & Partial<ConnectedProps>;
export declare type AdapterPoolConnected<ConnectedProps extends AdapterPoolConnectedProps> = AdapterPoolPrimalProps & ConnectedProps;
export declare type MAINTENANCE_MODE = typeof MAINTENANCE_MODE_AUTO | typeof MAINTENANCE_MODE_MANUAL;
export declare type ImportOptions = {
    byteOffset: number;
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ImportEventsStream = stream.Writable & {
    readonly byteOffset: number;
    readonly savedEventsCount: number;
};
export declare type ExportOptions = {
    cursor: Cursor;
    maintenanceMode: MAINTENANCE_MODE;
    bufferSize: number;
};
export declare type ExportEventsStream = stream.Readable & {
    readonly cursor: Cursor;
    readonly isBufferOverflow: boolean;
    readonly isEnd: boolean;
};
export declare type ImportSecretsOptions = {
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ExportSecretsOptions = {
    idx: number | null;
    maintenanceMode: MAINTENANCE_MODE;
};
export interface CommonAdapterFunctions<ConnectedProps extends AdapterPoolConnectedProps> {
    maybeThrowResourceError: CheckForResourceError;
    validateEventFilter: ValidateEventFilter;
    loadEvents: PoolMethod<ConnectedProps, Adapter['loadEvents']>;
    importEventsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['importEvents']>;
    exportEventsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['exportEvents']>;
    incrementalImport: PoolMethod<ConnectedProps, Adapter['incrementalImport']>;
    getNextCursor: GetNextCursor;
    importSecretsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['importSecrets']>;
    exportSecretsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['exportSecrets']>;
    init: PoolMethod<ConnectedProps, Adapter['init']>;
    drop: PoolMethod<ConnectedProps, Adapter['drop']>;
    gatherSecretsFromEvents: PoolMethod<ConnectedProps, Adapter['gatherSecretsFromEvents']>;
}
export interface AdapterFunctions<ConnectedProps extends AdapterPoolConnectedProps, ConnectionDependencies extends any, Config extends AdapterConfig> {
    beginIncrementalImport: PoolMethod<ConnectedProps, Adapter['beginIncrementalImport']>;
    commitIncrementalImport: PoolMethod<ConnectedProps, Adapter['commitIncrementalImport']>;
    connect: (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, connectionDependencies: ConnectionDependencies, config: Config) => Promise<void>;
    dispose: PoolMethod<ConnectedProps, Adapter['dispose']>;
    dropSnapshot: PoolMethod<ConnectedProps, Adapter['dropSnapshot']>;
    freeze: PoolMethod<ConnectedProps, Adapter['freeze']>;
    getLatestEvent: PoolMethod<ConnectedProps, Adapter['getLatestEvent']>;
    injectEvent: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['injectEvent']>;
    injectEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['injectEvents']>;
    loadEventsByCursor: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['loadEventsByCursor']>;
    loadEventsByTimestamp: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['loadEventsByTimestamp']>;
    loadSnapshot: PoolMethod<ConnectedProps, Adapter['loadSnapshot']>;
    pushIncrementalImport: PoolMethod<ConnectedProps, Adapter['pushIncrementalImport']>;
    rollbackIncrementalImport: PoolMethod<ConnectedProps, Adapter['rollbackIncrementalImport']>;
    saveEvent: PoolMethod<ConnectedProps, Adapter['saveEvent']>;
    saveSnapshot: PoolMethod<ConnectedProps, Adapter['saveSnapshot']>;
    shapeEvent: ShapeEvent;
    unfreeze: PoolMethod<ConnectedProps, Adapter['unfreeze']>;
    getSecret: PoolMethod<ConnectedProps, GetSecret>;
    setSecret: PoolMethod<ConnectedProps, SetSecret>;
    deleteSecret: PoolMethod<ConnectedProps, DeleteSecret>;
    loadSecrets?: PoolMethod<ConnectedProps, Adapter['loadSecrets']>;
    injectSecret?: PoolMethod<ConnectedProps, NonNullable<AdapterPoolConnectedProps['injectSecret']>>;
    initEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initEvents']>;
    initSecrets: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initSecrets']>;
    initFinal: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initFinal']>;
    dropEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropEvents']>;
    dropSecrets: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropSecrets']>;
    dropFinal: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropFinal']>;
    ensureEventSubscriber: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['ensureEventSubscriber']>;
    removeEventSubscriber: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['removeEventSubscriber']>;
    getEventSubscribers: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['getEventSubscribers']>;
    replicateEvents?: PoolMethod<ConnectedProps, Adapter['replicateEvents']>;
    replicateSecrets?: PoolMethod<ConnectedProps, Adapter['replicateSecrets']>;
    setReplicationStatus?: PoolMethod<ConnectedProps, Adapter['setReplicationStatus']>;
    setReplicationIterator?: PoolMethod<ConnectedProps, Adapter['setReplicationIterator']>;
    setReplicationPaused?: PoolMethod<ConnectedProps, Adapter['setReplicationPaused']>;
    getReplicationState?: PoolMethod<ConnectedProps, Adapter['getReplicationState']>;
    resetReplication?: PoolMethod<ConnectedProps, Adapter['resetReplication']>;
    getCursorUntilEventTypes?: PoolMethod<ConnectedProps, Adapter['getCursorUntilEventTypes']>;
    describe: PoolMethod<ConnectedProps, Adapter['describe']>;
    establishTimeLimit?: UnconnectedPoolMethod<ConnectedProps, Adapter['establishTimeLimit']>;
}
export interface Adapter {
    loadEvents: (filter: EventFilter) => Promise<EventsWithCursor>;
    importEvents: (options?: Partial<ImportOptions>) => ImportEventsStream;
    exportEvents: (options?: Partial<ExportOptions>) => ExportEventsStream;
    getLatestEvent: (filter: LatestEventFilter) => Promise<SavedEvent | null>;
    saveEvent: (event: InputEvent) => Promise<EventWithCursor>;
    init: () => Promise<void>;
    drop: () => Promise<void>;
    dispose: () => Promise<void>;
    freeze: () => Promise<void>;
    unfreeze: () => Promise<void>;
    getNextCursor: GetNextCursor;
    getSecretsManager: () => Promise<SecretsManager>;
    loadSnapshot: (snapshotKey: string) => Promise<string | null>;
    saveSnapshot: (snapshotKey: string, content: string) => Promise<void>;
    dropSnapshot: (snapshotKey: string) => Promise<void>;
    pushIncrementalImport: (events: VersionlessEvent[], importId: string) => Promise<void>;
    beginIncrementalImport: () => Promise<string>;
    commitIncrementalImport: (importId: string, validateAfterCommit?: any) => Promise<void>;
    rollbackIncrementalImport: () => Promise<void>;
    incrementalImport: (events: VersionlessEvent[]) => Promise<void>;
    loadSecrets: (filter: SecretFilter) => Promise<SecretsWithIdx>;
    importSecrets: (options?: Partial<ImportSecretsOptions>) => stream.Writable;
    exportSecrets: (options?: Partial<ExportSecretsOptions>) => stream.Readable;
    ensureEventSubscriber: (params: {
        applicationName: string;
        eventSubscriber: string;
        destination?: any;
        status?: any;
        updateOnly?: boolean;
    }) => Promise<boolean>;
    removeEventSubscriber: (params: {
        applicationName: string;
        eventSubscriber: string;
    }) => Promise<void>;
    getEventSubscribers: (params?: {
        applicationName?: string;
        eventSubscriber?: string;
    } | undefined) => Promise<Array<{
        applicationName: string;
        eventSubscriber: string;
        destination: any;
        status: any;
    }>>;
    gatherSecretsFromEvents: (events: SavedEvent[]) => Promise<GatheredSecrets>;
    replicateEvents: (events: OldEvent[]) => Promise<void>;
    replicateSecrets: (existingSecrets: OldSecretRecord[], deletedSecrets: Array<OldSecretRecord['id']>) => Promise<void>;
    setReplicationIterator: (iterator: SerializableMap) => Promise<void>;
    setReplicationStatus: (status: ReplicationStatus, info?: ReplicationState['statusData'], lastEvent?: OldEvent) => Promise<void>;
    setReplicationPaused: (pause: boolean) => Promise<void>;
    getReplicationState: () => Promise<ReplicationState>;
    resetReplication: () => Promise<void>;
    getCursorUntilEventTypes: (cursor: Cursor, untilEventTypes: Array<InputEvent['type']>) => Promise<string>;
    describe: () => Promise<EventStoreDescription>;
    establishTimeLimit: (getVacantTimeInMillis: () => number) => void;
}
export {};
